@@ -5,6 +5,7 @@ path.py
 A module designed to work with vector paths such as
 those stored in a DXF or SVG file.
 """
+
 import collections
 import copy
 from hashlib import sha256
@@ -17,7 +18,7 @@ from ..constants import log
 from ..constants import tol_path as tol
 from ..geometry import plane_transform
 from ..points import plane_fit
-from ..typed import Dict, List, NDArray, Optional, float64
+from ..typed import Dict, Iterable, List, NDArray, Optional, float64
 from ..visual import to_rgba
 from . import (
     creation,  # NOQA
@@ -71,7 +72,7 @@ class Path(parent.Geometry):
 
     def __init__(
         self,
-        entities: Optional[List[Entity]] = None,
+        entities: Optional[Iterable[Entity]] = None,
         vertices: Optional[NDArray[float64]] = None,
         metadata: Optional[Dict] = None,
         process: bool = True,
@@ -98,26 +99,24 @@ class Path(parent.Geometry):
 
         # assign each color to each entity
         self.colors = colors
-        # collect metadata into new dictionary
+        # collect metadata
         self.metadata = {}
-        if metadata.__class__.__name__ == "dict":
+        if isinstance(metadata, dict):
             self.metadata.update(metadata)
 
         # cache will dump whenever self.crc changes
         self._cache = caching.Cache(id_function=self.__hash__)
 
         if process:
-            # literally nothing will work if vertices
-            # aren't merged properly
+            # if our input had disconnected but identical points
+            # pretty much nothing will work if vertices aren't merged properly
             self.merge_vertices()
 
     def __repr__(self):
         """
         Print a quick summary of the number of vertices and entities.
         """
-        return "<trimesh.{}(vertices.shape={}, len(entities)={})>".format(
-            type(self).__name__, self.vertices.shape, len(self.entities)
-        )
+        return f"<trimesh.{type(self).__name__}(vertices.shape={self.vertices.shape}, len(entities)={len(self.entities)})>"
 
     def process(self):
         """
@@ -267,20 +266,6 @@ class Path(parent.Geometry):
         kdtree = cKDTree(self.vertices.view(np.ndarray))
         return kdtree
 
-    @property
-    def scale(self):
-        """
-        What is a representitive number that reflects the magnitude
-        of the world holding the paths, for numerical comparisons.
-
-        Returns
-        ----------
-        scale : float
-          Approximate size of the world holding this path
-        """
-        # return the diagonal length of the AABB
-        return np.linalg.norm(self.vertices.ptp(axis=0))
-
     @caching.cache_decorator
     def length(self):
         """
@@ -328,8 +313,7 @@ class Path(parent.Geometry):
         centroid : (d,) float
           Approximate centroid of the path
         """
-        centroid = self.bounds.mean(axis=0)
-        return centroid
+        return self.bounds.mean(axis=0)
 
     @property
     def extents(self):
@@ -1083,7 +1067,7 @@ class Path2D(Path):
         return path_3D
 
     @caching.cache_decorator
-    def polygons_closed(self):
+    def polygons_closed(self) -> NDArray:
         """
         Cycles in the vertex graph, as shapely.geometry.Polygons.
         These are polygon objects for every closed circuit, with no notion
@@ -1092,14 +1076,14 @@ class Path2D(Path):
 
         Returns
         ---------
-        polygons_closed: (n,) list of shapely.geometry.Polygon objects
+        polygons_closed : (n,) list of shapely.geometry.Polygon objects
         """
         # will attempt to recover invalid garbage geometry
         # and will be None if geometry is unrecoverable
         return polygons.paths_to_polygons(self.discrete)
 
     @caching.cache_decorator
-    def polygons_full(self):
+    def polygons_full(self) -> List:
         """
         A list of shapely.geometry.Polygon objects with interiors created
         by checking which closed polygons enclose which other polygons.
